# -- coding: utf-8 --

import os
import sys
import runpy
from datetime import timedelta as delta

from settings import *
from src.extractor import extractor
from src.calculator import calculator
from src.drawers import *


### Defines ###
DATA_FOLDER = 'data/'
EXT_OUTPUT_FILENAME = 'all_selected_donors.stpy'
CALC_OUTPUT_FILENAME = 'groups.gpy'
CALC_STATS_FILENAME = 'stats.py'
CALC_OUTLIERS_FILENAME = 'outliers.stpy'


class Experiment():
    id = 'GenericGlobalExperiment'
    gap_min_days = 0
    gap_max_days = INF
    group_interval = INF
    highest_donation_class = 2

    # list of function to draw alpha calculations
    drawers = [
        alphas_ordered, alphas_ordered_yearly, alphas_histogram,
        alphas_classification_histogram, alphas_normal_fit,

        sexes_ordered, sexes_ordered_yearly,
        sexes_histograms, sexes_boxplot, sexes_boxplot_yearly,
        sexes_classification_histograms, sexes_normal_fit,
        sexes_comparison,

        classes_ordered, classes_ordered_yearly,
        classes_histograms, classes_boxplot, classes_boxplot_yearly,
        classes_classification_histograms, classes_normal_fit,
        classes_comparison,

        anemics_ordered, anemics_ordered_yearly, anemics_histogram,
        before_anemia_ordered, before_anemia_ordered_yearly, before_anemia_histogram,
        after_anemia_ordered, after_anemia_ordered_yearly, after_anemia_histogram,
        not_anemics_ordered, not_anemics_ordered_yearly, not_anemics_histogram,
        anemia_classes_comparison, anemia_boxplot,

        stats_tex_table,
    ]

    def __init__(self):
        my_folder = os.path.join(DATA_FOLDER, 'exp_' + str(self.id))
        folders = {
            'extracted': DATA_FOLDER,
            'home': my_folder,
            'positive_outliers': os.path.join(my_folder, 'outliers/positive'),
            'negative_outliers': os.path.join(my_folder, 'outliers/negative')
        }
        for folder_path in folders.itervalues():
            if not os.path.exists(folder_path):
                os.makedirs(folder_path)
        files = {
            'extracted': os.path.join(folders['extracted'], EXT_OUTPUT_FILENAME),
            'calculated': os.path.join(folders['home'], CALC_OUTPUT_FILENAME),
            'stats': os.path.join(folders['home'], CALC_STATS_FILENAME),
            'positive_outliers': os.path.join(folders['positive_outliers'], CALC_OUTLIERS_FILENAME),
            'positive_outliers': os.path.join(folders['negative_outliers'], CALC_OUTLIERS_FILENAME),
        }
        self.paths = {'folders': folders, 'files': files}

    def _extract(self):
        extractor(self.paths['files']['extracted'])

    def _calculate(self):
        if os.path.exists(self.paths['files']['calculated']):
            return
        
        group_definition = {
            # avoids degenerate cases of alphas
            'gap_min_days': self.gap_min_days,
            # in a group definition is the delta parameter
            'gap_max_days': self.gap_max_days,
            # in a group definition is the Delta parameter
            'group_interval': self.group_interval,
            # number of classes you want
            'highest_donation_class': self.highest_donation_class
        }
        calculator(group_definition, self.paths)

    def _graph(self):
        for d in self.drawers:
            img_file = d(self.paths)
            
            print "Generated", img_file

    def run(self):
        self._extract()
        self._calculate()
        self._graph()

