# -- coding: utf-8 --

import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt

from settings import *
from src.parsers import GpyParser

from alphas import AlphasDrawerComposition, AlphasDrawer
from alphas import AlphasHistogramComposition, AlphasHistogram
from alphas import AlphasClassificationHistogram
from alphas import AlphasNormalFit
from alphas import AlphasBoxplotComposition
from stats import StatsDrawer


### alphas drawers ###
def alphas_ordered(paths):
    parser = GpyParser(paths['files']['calculated'])
    alphas = [ a for a in parser.alphas() ]
    return AlphasDrawer(paths, alphas, 'alphas').ordered().image()

def alphas_ordered_yearly(paths):
    parser = GpyParser(paths['files']['calculated'])
    alphas = [ a for a in parser.alphas() ]
    return AlphasDrawer(paths, alphas, 'alphas(yearly)').yearly().ordered().image()

def alphas_histogram(paths):
    parser = GpyParser(paths['files']['calculated'])
    alphas = [ a for a in parser.alphas() ]
    return AlphasHistogram(paths, alphas, 'alphas_histogram').image()

def alphas_classification_histogram(paths):
    parser = GpyParser(paths['files']['calculated'])
    alphas = [ a for a in parser.alphas() ]
    return AlphasClassificationHistogram(paths, alphas,
                'alphas_classification').image()

def alphas_normal_fit(paths):
    parser = GpyParser(paths['files']['calculated'])
    alphas = [ a for a in parser.alphas() ]
    return AlphasNormalFit(paths, alphas, 'alphas_normal_fit').image()

### sexes drawers ###
def sexes_ordered(paths):
    parser = GpyParser(paths['files']['calculated'])
    sexes = AlphasDrawerComposition(paths, "sexes")
    filepattern = "sex_{id}"
    alphas_men = [ a for a in parser.alphas(use_men=True, use_women=False) ]
    sexes.add(AlphasDrawer(paths, alphas_men, filepattern, id="M"))
    alphas_women = [ a for a in parser.alphas(use_men=False, use_women=True) ]
    sexes.add(AlphasDrawer(paths, alphas_women, filepattern, id="F", format='r'))
    return sexes.ordered().image()

def sexes_ordered_yearly(paths):
    parser = GpyParser(paths['files']['calculated'])
    sexes = AlphasDrawerComposition(paths, "sexes(yearly)")
    filepattern = "sex_{id}(yearly)"
    alphas_men = [ a for a in parser.alphas(use_men=True, use_women=False) ]
    sexes.add(AlphasDrawer(paths, alphas_men, filepattern, id="M"))
    alphas_women = [ a for a in parser.alphas(use_men=False, use_women=True) ]
    sexes.add(AlphasDrawer(paths, alphas_women, filepattern, id="F", format='r'))
    return sexes.yearly().ordered().image()

def sexes_histograms(paths):
    parser = GpyParser(paths['files']['calculated'])
    sexes = AlphasHistogramComposition(paths, "sexes_histogram")
    filepattern = "sex_histogram_{id}"
    alphas_men = [ a for a in parser.alphas(use_men=True, use_women=False) ]
    sexes.add(AlphasHistogram(paths, alphas_men, filepattern, id="M"))
    alphas_women = [ a for a in parser.alphas(use_men=False, use_women=True) ]
    sexes.add(AlphasHistogram(paths, alphas_women, filepattern, id="F"))
    return sexes.ordered().images()

def sexes_comparison(paths):
    parser = GpyParser(paths['files']['calculated'])
    sexes = AlphasHistogramComposition(paths, "sexes_comparison")
    filepattern = "sex_comparison_{id}"
    alphas_men = [ a for a in parser.alphas(use_men=True, use_women=False) ]
    sexes.add(AlphasHistogram(paths, alphas_men, filepattern, id="M"))
    alphas_women = [ a for a in parser.alphas(use_men=False, use_women=True) ]
    sexes.add(AlphasHistogram(paths, alphas_women, filepattern, id="F", format='r'))
    return sexes.ordered().image()

def sexes_classification_histograms(paths):
    parser = GpyParser(paths['files']['calculated'])
    sexes = AlphasHistogramComposition(paths, "sexes_classification")
    filepattern = "sex_classification_{id}"
    alphas_men = [ a for a in parser.alphas(use_men=True, use_women=False) ]
    sexes.add(AlphasClassificationHistogram(paths, alphas_men, filepattern, id="M"))
    alphas_women = [ a for a in parser.alphas(use_men=False, use_women=True) ]
    sexes.add(AlphasClassificationHistogram(paths, alphas_women, filepattern, id="F"))
    return sexes.ordered().images()

def sexes_normal_fit(paths):
    parser = GpyParser(paths['files']['calculated'])
    sexes = AlphasHistogramComposition(paths, "sexes_normal_fit")
    filepattern = "sex_normal_fit_{id}"
    alphas_men = [ a for a in parser.alphas(use_men=True, use_women=False) ]
    sexes.add(AlphasNormalFit(paths, alphas_men, filepattern, id="M"))
    alphas_women = [ a for a in parser.alphas(use_men=False, use_women=True) ]
    sexes.add(AlphasNormalFit(paths, alphas_women, filepattern, id="F"))
    return sexes.ordered().images()

def sexes_boxplot(paths):
    parser = GpyParser(paths['files']['calculated'])
    sexes = AlphasBoxplotComposition(paths, "sexes_boxplot")
    filepattern = "sex_boxplot_{id}"
    alphas_men = [ a for a in parser.alphas(use_men=True, use_women=False) ]
    sexes.add(AlphasHistogram(paths, alphas_men, filepattern, id=0))
    alphas_women = [ a for a in parser.alphas(use_men=False, use_women=True) ]
    sexes.add(AlphasDrawer(paths, alphas_women, filepattern, id=1, format='r'))
    return sexes.ordered().image()

def sexes_boxplot_yearly(paths):
    parser = GpyParser(paths['files']['calculated'])
    sexes = AlphasBoxplotComposition(paths, "sexes_boxplot(yearly)")
    filepattern = "sex_boxplot_{id}(yearly)"
    alphas_men = [ a for a in parser.alphas(use_men=True, use_women=False) ]
    sexes.add(AlphasHistogram(paths, alphas_men, filepattern, id=0))
    alphas_women = [ a for a in parser.alphas(use_men=False, use_women=True) ]
    sexes.add(AlphasDrawer(paths, alphas_women, filepattern, id=1, format='r'))
    return sexes.yearly().ordered().image()

### classes drawers ###
def classes_ordered(paths):
    parser = GpyParser(paths['files']['calculated'])
    classes = AlphasDrawerComposition(paths, "classes")
    for class_num, group_dicts in parser.classes():
        alphas = [ gd['pol'][0] for gd in group_dicts ]
        filepattern = "class_{id}"
        ad = AlphasDrawer(paths, alphas, filepattern, id=class_num)
        classes.add(ad)
    return classes.ordered().images()

def classes_ordered_yearly(paths):
    parser = GpyParser(paths['files']['calculated'])
    classes = AlphasDrawerComposition(paths, "classes(yearly)")
    for class_num, group_dicts in parser.classes():
        alphas = [ gd['pol'][0] for gd in group_dicts ]
        filepattern = "class_{id}(yearly)"
        ad = AlphasDrawer(paths, alphas, filepattern, id=class_num)
        classes.add(ad)
    return classes.yearly().ordered().images()

def classes_histograms(paths):
    parser = GpyParser(paths['files']['calculated'])
    classes = AlphasHistogramComposition(paths, "classes_histogram")
    for class_num, group_dicts in parser.classes():
        alphas = [ gd['pol'][0] for gd in group_dicts ]
        filepattern = "class_histogram_{id}"
        ad = AlphasHistogram(paths, alphas, filepattern, id=class_num)
        classes.add(ad)
    return classes.ordered().images()

def classes_comparison(paths):
    parser = GpyParser(paths['files']['calculated'])
    classes = AlphasHistogramComposition(paths, "classes_comparison")
    for class_num, group_dicts in parser.classes():
        alphas = [ gd['pol'][0] for gd in group_dicts ]
        filepattern = "class_comparison_{id}"
        ad = AlphasHistogram(paths, alphas, filepattern, id=class_num)
        classes.add(ad)
    return classes.ordered().image()

def classes_classification_histograms(paths):
    parser = GpyParser(paths['files']['calculated'])
    classes = AlphasHistogramComposition(paths, "classes_classification")
    for class_num, group_dicts in parser.classes():
        alphas = [ gd['pol'][0] for gd in group_dicts ]
        filepattern = "class_classification_{id}"
        ad = AlphasClassificationHistogram(paths, alphas, filepattern, id=class_num)
        classes.add(ad)
    return classes.ordered().images()

def classes_normal_fit(paths):
    parser = GpyParser(paths['files']['calculated'])
    classes = AlphasHistogramComposition(paths, "classes_normal_fit")
    for class_num, group_dicts in parser.classes():
        alphas = [ gd['pol'][0] for gd in group_dicts ]
        filepattern = "class_normal_fit_{id}"
        ad = AlphasNormalFit(paths, alphas, filepattern, id=class_num)
        classes.add(ad)
    return classes.ordered().images()

def classes_boxplot(paths):
    parser = GpyParser(paths['files']['calculated'])
    classes = AlphasBoxplotComposition(paths, "classes_boxplot")
    for class_num, group_dicts in parser.classes():
        alphas = [ gd['pol'][0] for gd in group_dicts ]
        filepattern = "class_boxplot_{id}"
        ad = AlphasDrawer(paths, alphas, filepattern, id=class_num)
        classes.add(ad)
    return classes.ordered().image()

def classes_boxplot_yearly(paths):
    parser = GpyParser(paths['files']['calculated'])
    classes = AlphasBoxplotComposition(paths, "classes_boxplot(yearly)")
    for class_num, group_dicts in parser.classes():
        alphas = [ gd['pol'][0] for gd in group_dicts ]
        filepattern = "class_boxplot_{id}(yearly)"
        ad = AlphasDrawer(paths, alphas, filepattern, id=class_num)
        classes.add(ad)
    return classes.yearly().ordered().image()

### anemic drawers ###
def anemics_ordered(paths):
    parser = GpyParser(paths['files']['calculated'])
    alphas = [ a for a in parser.alphas(use_never_anemic=False,
                use_before_anemia=True, use_after_anemia=True) ]
    return AlphasDrawer(paths, alphas, 'anemics').ordered().image()

def anemics_ordered_yearly(paths):
    parser = GpyParser(paths['files']['calculated'])
    alphas = [ a for a in parser.alphas(use_never_anemic=False,
                use_before_anemia=True, use_after_anemia=True) ]
    return AlphasDrawer(paths, alphas, 'anemics(yearly)').yearly().ordered().image()

def anemics_histogram(paths):
    parser = GpyParser(paths['files']['calculated'])
    alphas = [ a for a in parser.alphas(use_never_anemic=False,
                use_before_anemia=True, use_after_anemia=True) ]
    return AlphasHistogram(paths, alphas, 'anemics_histogram').image()

def before_anemia_ordered(paths):
    parser = GpyParser(paths['files']['calculated'])
    alphas = [ a for a in parser.alphas(use_never_anemic=False,
                use_before_anemia=True, use_after_anemia=False) ]
    return AlphasDrawer(paths, alphas, 'before_anemia').ordered().image()

def before_anemia_ordered_yearly(paths):
    parser = GpyParser(paths['files']['calculated'])
    alphas = [ a for a in parser.alphas(use_never_anemic=False,
                use_before_anemia=True, use_after_anemia=False) ]
    return AlphasDrawer(paths, alphas, 'before_anemia(yearly)').yearly().ordered().image()

def before_anemia_histogram(paths):
    parser = GpyParser(paths['files']['calculated'])
    alphas = [ a for a in parser.alphas(use_never_anemic=False,
                use_before_anemia=True, use_after_anemia=False) ]
    return AlphasHistogram(paths, alphas, 'before_anemia_histogram').image()

def after_anemia_ordered(paths):
    parser = GpyParser(paths['files']['calculated'])
    alphas = [ a for a in parser.alphas(use_never_anemic=False,
                use_before_anemia=False, use_after_anemia=True) ]
    return AlphasDrawer(paths, alphas, 'after_anemia').ordered().image()

def after_anemia_ordered_yearly(paths):
    parser = GpyParser(paths['files']['calculated'])
    alphas = [ a for a in parser.alphas(use_never_anemic=False,
                use_before_anemia=False, use_after_anemia=True) ]
    return AlphasDrawer(paths, alphas, 'after_anemia(yearly)').yearly().ordered().image()

def after_anemia_histogram(paths):
    parser = GpyParser(paths['files']['calculated'])
    alphas = [ a for a in parser.alphas(use_never_anemic=False,
                use_before_anemia=False, use_after_anemia=True) ]
    return AlphasHistogram(paths, alphas, 'after_anemia_histogram').image()

def not_anemics_ordered(paths):
    parser = GpyParser(paths['files']['calculated'])
    alphas = [ a for a in parser.alphas(use_never_anemic=True,
                use_before_anemia=False, use_after_anemia=False) ]
    return AlphasDrawer(paths, alphas, 'not_anemics').ordered().image()

def not_anemics_ordered_yearly(paths):
    parser = GpyParser(paths['files']['calculated'])
    alphas = [ a for a in parser.alphas(use_never_anemic=True,
                use_before_anemia=False, use_after_anemia=False) ]
    return AlphasDrawer(paths, alphas, 'not_anemics(yearly)').yearly().ordered().image()

def not_anemics_histogram(paths):
    parser = GpyParser(paths['files']['calculated'])
    alphas = [ a for a in parser.alphas(use_never_anemic=True,
                use_before_anemia=False, use_after_anemia=False) ]
    return AlphasHistogram(paths, alphas, 'not_anemics_histogram').image()

def anemia_classes_comparison(paths):
    parser = GpyParser(paths['files']['calculated'])
    anemia = AlphasHistogramComposition(paths, "anemia_comparison")
    filepattern = "anemia_comparison_{id}"
    alphas_never = [ a for a in parser.alphas(use_never_anemic=True,
                        use_before_anemia=False, use_after_anemia=False) ]
    anemia.add(AlphasHistogram(paths, alphas_never, filepattern, id="Not anemic", format="r"))
    alphas_before = [ a for a in parser.alphas(use_never_anemic=False,
                        use_before_anemia=True, use_after_anemia=False) ]
    anemia.add(AlphasHistogram(paths, alphas_before, filepattern, id="Before anemia"))
    alphas_after = [ a for a in parser.alphas(use_never_anemic=False,
                        use_before_anemia=False, use_after_anemia=True) ]
    anemia.add(AlphasHistogram(paths, alphas_after, filepattern, id="After anemia"))
    return anemia.ordered().image()

def anemia_boxplot(paths):
    parser = GpyParser(paths['files']['calculated'])
    anemia = AlphasBoxplotComposition(paths, "anemia_boxplot")
    filepattern = "anemia_boxplot_{id}"
    alphas_never = [ a for a in parser.alphas(use_never_anemic=True,
                        use_before_anemia=False, use_after_anemia=False) ]
    anemia.add(AlphasHistogram(paths, alphas_never, filepattern, id=0))
    alphas_before = [ a for a in parser.alphas(use_never_anemic=False,
                        use_before_anemia=True, use_after_anemia=False) ]
    anemia.add(AlphasHistogram(paths, alphas_before, filepattern, id=1))
    alphas_after = [ a for a in parser.alphas(use_never_anemic=False,
                        use_before_anemia=False, use_after_anemia=True) ]
    anemia.add(AlphasHistogram(paths, alphas_after, filepattern, id=2))
    return anemia.ordered().image()

### others ###
def stats_tex_table(paths):
    return StatsDrawer(paths).write()
