# -- coding: utf-8 --

from __future__ import division

from settings import *
from src.parsers import StatsParser

from drawer import Drawer

class StatsDrawer(Drawer):

    def __init__(self, paths):
        super(StatsDrawer, self).__init__(paths, "stats.tex")
        parser = StatsParser(paths['files']['stats'])

        self.ldicts = []
        self.total_groups = 0
        for d in parser.stat_dicts():
            self.ldicts.append(d)
            self.total_groups += d['num_groups']

    def write(self):
        filestring = \
        "\\begin{tabular}{ | c | c | c | c | } \\hline\n" \
        "Doações por grupo & Número de grupos (\\%) & Média $(*10^3)$ & Desvio Padrão $(*10^3)$  \\\\ \\hline \n"
        for d in self.ldicts:
            d['sign'] = "+" if (id(d) == id(self.ldicts[-1])) else "" # "+" at last iteraction
            d['perc'] = "%.2f" % (d['num_groups'] * 100 / self.total_groups)
            d['mean'] = d['mean'] * 1000
            d['deviation'] = d['deviation'] * 1000

            s = "{num_donations}{sign} & {num_groups} ({perc}\\%%) & {mean} & {deviation} \\\\ \hline\n"
            filestring += s.format(**d) % d
        filestring += "\\end{tabular}\n"

        f = open(self.filepath, 'w')
        f.write(filestring)
        f.close()

        return self.filepath