# -- coding: utf-8 --
""" Script usado para ler e iterar sobre arquivos no formato stpy e gpy.
"""

import sys
import re
import datetime
from numpy import array

from db import *


class Parser():

    has_meta = True

    def __init__(self, filepath):
        self.filepath = filepath
        self.open()

    def open(self):
        self.f = open(self.filepath, 'r')
        if self.has_meta:
            first_line = self.f.readline()
            self.meta = eval(first_line)

    def close(self):
        self.f.close()

    def reset(self):
        try:
            self.close()
        except:
            pass
        self.open()
        

class StpyParser(Parser):
    meta = {}

    def donors(self):
        '''Devolve próximo doador não processado'''
        for l in self.f:
            donor_dict = eval(l)
            d = Donor(donor_dict['donor_id'])
            d.set_time_series(donor_dict['donations'])
            
            yield d
        self.close()

    def donor(self, id):
        '''Devolve um doador específico'''
        d = None
        for l in self.f:
            donor_dict = eval(l)
            if donor_dict['donor_id'] == id:
                d = Donor(donor_dict['donor_id'])
                d.set_time_series(donor_dict['donations'])
                break
        self.close()
        self.open()
        return d


class GpyParser(Parser):
    meta = {}

    def _is_neg_outlier(self, alpha):
        return True if alpha < self.meta['negative_threshold'] else False

    def _is_pos_outlier(self, alpha):
        return True if self.meta['positive_threshold'] < alpha else False

    def _is_valid(self, alpha):
        if self._is_neg_outlier(alpha) or self._is_pos_outlier(alpha):
            return False
        return True

    def _is_man(self, sex):
        return True if sex == 'M' else False

    def _is_woman(self, sex):
        return True if sex == 'F' else False

    def _never_anemic(self, anemic_related):
        return True if anemic_related == 'never' else False

    def _before_anemia(self, anemic_related):
        return True if anemic_related == 'before' else False

    def _after_anemia(self, anemic_related):
        return True if anemic_related == 'after' else False

    def _iterator(self, use_valids=True, use_positive_outliers=True,
        use_negative_outliers=True, use_men=True, use_women=True,
        use_never_anemic=True, use_before_anemia=True, use_after_anemia=True):
        for l in self.f:
            group_dict = eval(l)
            alpha = group_dict['pol'][0]
            if (use_men and self._is_man(group_dict['sex'])) \
            or (use_women and self._is_woman(group_dict['sex'])):
                if (use_valids and self._is_valid(alpha)) \
                or (use_positive_outliers and self._is_pos_outlier(alpha)) \
                or (use_negative_outliers and self._is_neg_outlier(alpha)):
                    if (use_never_anemic and self._never_anemic(group_dict['anemia_related'])) \
                    or (use_before_anemia and self._before_anemia(group_dict['anemia_related'])) \
                    or (use_after_anemia and self._after_anemia(group_dict['anemia_related'])):
                        yield group_dict
        self.close()
        self.open()

    def alphas(self, **kw):
        '''Devolve próximo alpha válido não processado'''
        for group_dict in self._iterator(**kw):
            alpha = group_dict['pol'][0]
            yield alpha

    def groups(self, **kw):
        '''Devolve todas as informações do próximo grupo não processado'''
        for group_dict in self._iterator(**kw):
            yield group_dict

    def classes(self, **kw):
        '''Devolve estrutura para processar alphas da próxima classe ainda não
        processada'''
        class_num = -1
        class_group_dicts = []

        for group_dict in self._iterator(**kw):
            if group_dict['num_donations'] != class_num:
                # cacheou uma classe e vai começar outra
                if class_group_dicts:
                    yield (class_num, class_group_dicts)

                class_num = group_dict['num_donations']
                class_group_dicts = []

            class_group_dicts.append(group_dict)
        yield (class_num, class_group_dicts)
        self.close()

class StatsParser(Parser):

    has_meta = False

    def stat_dicts(self):
        for l in self.f:
            d = eval(l)
            yield d
        self.close()
